/*
	Copyright (C) 2011 Steve Thomas <steve AT tobtu DOT com>

	This file is part of MySQL323 Collider.

	MySQL323 Collider is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	MySQL323 Collider is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with MySQL323 Collider.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MYSQL323_THREAD_H
#define MYSQL323_THREAD_H

#include "thread.h"
#include "workqueue.h"
#include "keyspace.h"
#include "atomic.h"
#include <vector>
#include <string>
#include <list>

using namespace std;

struct passwordElement
{
	union
	{
		uint64_t hash;
		struct
		{
#ifdef ARC_LITTLE_ENDIAN
			uint32_t hash1, hash2;
#else
			uint32_t hash2, hash1;
#endif
		};
	};
	uint64_t pw;
};

struct mysqlHash
{
	uint32_t hash1, hash2;
};

struct mySQL323ThreadArgs
{
	WorkQueue *inQueue, *outQueue;
	const KeySpace *keySpace;
	const KeySpace *prefixKeySpace;
	Atomic *keySpacePos;

	passwordElement **indexedLookupTable;
	uint32_t *bitmap;
	uint8_t *prevHash;

	uint32_t threadId;
};

struct mySQL323WorkQueueData
{
	double time;
	list<pair<uint32_t,string> > foundPws;
	vector<mysqlHash> hashes;
	uint32_t sum, count, threadId;
};

struct mySQL323WorkQueueNode
{
	mySQL323WorkQueueNode *next;
	mySQL323WorkQueueData *data;
};

class MySQL323Thread : public Thread
{
public:
	MySQL323Thread();
	~MySQL323Thread();

	int stopRunning();
	void *debugrun(void *arg);

protected:
	void *threadFunc(void *arg);

private:
	WorkQueue *m_queue;
	MUTEX m_mutex;
};

#endif
